skeeter
=======

Relay messages between an MQTT message broker and OSC. 

This code base has been developed by [ZKM | Hertz-Lab](https://zkm.de/en/about-the-zkm/organization/hertz-lab) as part of the project [»The Intelligent Museum«](#the-intelligent-museum). 

Copyright (c) 2023 ZKM | Karlsruhe.  
Copyright (c) 2023 Dan Wilcox.  

BSD Simplified License.

Description
-----------

_To be filled out._

Dependencies
------------

* Python 3
* [paho-mqtt](https://github.com/eclipse/paho.mqtt.python)
* [python-osc](https://github.com/attwad/python-osc)

Setup
-----

Install Python 3, if not already available. For instance, on macOS using [Homebrew](http://brew.sh):

```shell
brew install python3
```

Create a virtual environment and install the script's dependencies:

```shell
make
```

Mosquitto
---------

Any MQTT message broker (server) should work with the sketter clients. We recommend trying [Mosquitto](https://mosquitto.org) for testing.

_Note: this project is in prelimary stages and does not use currently use authenticaton._

### macOS

Install using [homebrew](https://brew.sh) on macOS:

    brew install mosquitto

Edit and add to config /opt/homebrew/etc/mosquitto/mosquitto.conf:

~~~
listener 1883
protocol mqtt

listener 9001
protocol websockets

allow_anonymous true
~~~

Run directly:

    mosquitto -c /opt/homebrew/etc/mosquitto/mosquitto.conf

or start as a background service:

    brew services start mosquitto

### Linux

For Linux, use your distro's package manager, ala:

    sudo apt install mosquitto

Edit and add to config /etc/mosquitto/mosquitto.conf:

~~~
listener 1883
protocol mqtt

listener 9001
protocol websockets

allow_anonymous true
~~~

Run directly:

    mosquitto -c /etc/mosquitto/mosquitto.conf

Running
-------

_To be filled out._

**skeeter-send & skeeter-recv _may_ be merged into a single script.**

Example Clients
---------------

### Send & Receive

A set of example clients are included:

* `pd/sendclient.pd`: Pure Data patch which sends OSC messages
* `pd/recvclient.pd`: Pure Data patch which receives OSC messages

Both examples should work together with the default address & ports on the same localhost:

    pd/sendclient.pd --OSC-> skeeter-send.py --MQTT-> MQTT broker
    pd/recvclient.pd <-OSC-- skeeter-recv.py <-MQTT-- MQTT broker

First start thoscy-send or thoscy-recv, then start the affiliated client, ie. sendclient.pd & thoscy-send. To see data changing on the server, create/open a ThingsBoard Dashboard with widgets that show the device entity values.

### Web Dashboard

_Add HTML websocket client info too._

1. start mosquitto
2. run `pd/sendtemp.pd`: Pure Data patch which generates temperature messages
3. open `html/client/index.html` in a web browser

_Note: The example dashboard currently assumes mosquitto runs on localhost._

References
----------

Paho MQTT Python client:
* https://www.emqx.com/en/blog/comparision-of-python-mqtt-client
* https://github.com/eclipse/paho.mqtt.python

MQTT message formatting:
* https://www.hivemq.com/blog/mqtt-essentials-part-5-mqtt-topics-best-practices/
* https://www.hivemq.com/blog/mqtt5-essentials-part8-payload-format-description/

JSON as MQTT payload?
* https://techtutorialsx.com/2017/04/29/esp32-sending-json-messages-over-mqtt/

Websockets:
* http://www.steves-internet-guide.com/mqtt-websockets/

The Intelligent Museum
----------------------

An artistic-curatorial field of experimentation for deep learning and visitor participation

The [ZKM | Center for Art and Media](https://zkm.de/en) and the [Deutsches Museum Nuremberg](https://www.deutsches-museum.de/en/nuernberg/information/) cooperate with the goal of implementing an AI-supported exhibition. Together with researchers and international artists, new AI-based works of art will be realized during the next four years (2020-2023).  They will be embedded in the AI-supported exhibition in both houses. The Project „The Intelligent Museum” is funded by the Digital Culture Programme of the [Kulturstiftung des Bundes](https://www.kulturstiftung-des-bundes.de/en) (German Federal Cultural Foundation) and funded by the [Beauftragte der Bundesregierung für Kultur und Medien](https://www.bundesregierung.de/breg-de/bundesregierung/staatsministerin-fuer-kultur-und-medien) (Federal Government Commissioner for Culture and the Media).

As part of the project, digital curating will be critically examined using various approaches of digital art. Experimenting with new digital aesthetics and forms of expression enables new museum experiences and thus new ways of museum communication and visitor participation. The museum is transformed to a place of experience and critical exchange.
