// refs:
// * https://github.com/eclipse/paho.mqtt.javascript
// * https://www.d3indepth.com/selections/
// * https://www.d3indepth.com/shapes/
// * https://www.d3indepth.com/scales/

// ----- mqtt -----

// use localhost when only connecting internally
let host = "localhost"
let port = 9001
let topic = "test"

let client = new Paho.MQTT.Client(host, Number(port), "wsclient");
client.onConnectionLost = (response) => {
  if(response.errorCode !== 0) {
    console.log("mqtt: disconnected: ", response.errorMessage)
  }
}
client.onMessageArrived = (message) => {
  console.log("mqtt recv: ", message.payloadString)
  document.getElementById("message").innerText = message.payloadString
  let data = JSON.parse(message.payloadString)
  if(data) {
    if(data.temperature) {
      console.log("temp is " + data.temperature)
      temp.set(data.temperature)
      hist.push(data.temperature)
    }
  }
}
client.connect({onSuccess: () => {
  console.log("mqtt: connected")
  client.subscribe(topic)
}})

// ----- d3 -----

// temp knob and axis
let temp = {
    svg: null,
    circle: null,
    scale: {x: null},
    color: null,
    setup: function() {
      temp.svg = d3.select("#canvas")
      temp.scale.x = d3.scaleLinear()
        .domain([-20, 40]) // data min max
        .range([0, 400]) // pixel min max
        .clamp(true)
      temp.color = d3.scaleSequential()
        .domain([-20, 40]) // data min max
        .interpolator(d3.interpolateCool) // color output
      temp.svg.call(d3.axisBottom(temp.scale.x)) // set axis from domain
      temp.circle = temp.svg.append("circle")
        .attr("cx", temp.scale.x(0)).attr("cy", 0).attr("r", 8).style("fill", temp.color(0))
    },
    set: function(t) {
      if(temp.circle) {
        temp.circle.attr("cx", temp.scale.x(t))
        temp.circle.style("fill", temp.color(t))
      }
    }
}

// temp histogram
let hist = {
    data: [],
    linegen: null,
    path: null,
    scale: {x: null, y: null},
    setup: function() {
      hist.svg = d3.select("#canvas2")
      hist.scale.x = d3.scaleLinear().domain([0, 10]).range([0, 400]) // x domain: data len min max
      hist.scale.y = d3.scaleLinear().domain([40, -20]).range([0, 200]) // y domain: temp min max
      hist.linegen = d3.line()
      hist.linegen
        .curve(d3.curveCardinal)
        .x(function(d, i){return hist.scale.x(i)})
        .y(function(d){return hist.scale.y(d)})
      hist.path = hist.svg.append("path")
        .attr("d", hist.linegen(hist.data))
        .style("stroke", "orange")
        .style("stroke-width", 4)
      hist.svg.call(d3.axisLeft(hist.scale.y)) // set axis from domain
    },
    update: function() {
      hist.path.attr("d", hist.linegen(hist.data))
    },
    push: function(t) {
      hist.data.push(t)
      while(hist.data.length > 10) {
        hist.data.shift()
      }
      hist.update()
      console.log(hist.data)
    }
}

// setup
window.addEventListener("load", () => {
  temp.setup()
  hist.setup()
})
